@@ -41,6 +41,7 @@ from upath._stat import UPathStatResult
 from upath.registry import get_upath_class
 from upath.types import UNSET_DEFAULT
 from upath.types import JoinablePathLike
+from upath.types import OnNameCollisionFunc
 from upath.types import PathInfo
 from upath.types import ReadablePath
 from upath.types import ReadablePathLike
@@ -573,7 +574,7 @@ class _UPathMixin(metaclass=_UPathMeta):
         # FIXME: normalization needs to happen in unchain already...
         chain = Chain.from_list(Chain.from_list(segments).to_list())
         if len(args) > 1:
-            flavour = WrappedFileSystemFlavour.from_protocol(protocol)
+            flavour = WrappedFileSystemFlavour.from_protocol(chain.active_path_protocol)
             joined = flavour.join(chain.active_path, *args[1:])
             stripped = flavour.strip_protocol(joined)
             chain = chain.replace(path=stripped)
@@ -785,7 +786,7 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
     if TYPE_CHECKING:  # noqa: C901
         _chain: Chain
         _chain_parser: FSSpecChainParser
-        _fs_cached: bool
+        _fs_cached: AbstractFileSystem
         _raw_urlpaths: Sequence[JoinablePathLike]
         _relative_base: str | None
 
@@ -963,8 +964,10 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
         new_instance = type(self)(
             *pathsegments,
             protocol=self._protocol,
-            **self._storage_options,
+            **self.storage_options,
         )
+        if hasattr(self, "_fs_cached"):
+            new_instance._fs_cached = self._fs_cached
 
         if is_relative:
             new_instance._relative_base = self._relative_base
@@ -1088,7 +1091,7 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
                     self._relative_base,
                     str(self),
                     protocol=self._protocol,
-                    **self._storage_options,
+                    **self.storage_options,
                 )
                 parent = pth.parent
                 parent._relative_base = self._relative_base
@@ -1119,7 +1122,7 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
                     break
                 parent = parent.parent
                 parents.append(parent)
-            return parents
+            return tuple(parents)
         return super().parents
 
     def joinpath(self, *pathsegments: JoinablePathLike) -> Self:
@@ -1183,7 +1186,11 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
         base = self
         if self.parts[-1:] == ("",):
             base = self.parent
-        for name in base.fs.listdir(base.path):
+        fs = base.fs
+        base_path = base.path
+        if not fs.isdir(base_path):
+            raise NotADirectoryError(str(self))
+        for name in fs.listdir(base_path):
             # fsspec returns dictionaries
             if isinstance(name, dict):
                 name = name.get("name")
@@ -1192,7 +1199,7 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
                 continue
             # only want the path name with iterdir
             _, _, name = name.removesuffix(sep).rpartition(self.parser.sep)
-            yield base.with_segments(base.path, name)
+            yield base.with_segments(base_path, name)
 
     def __open_reader__(self) -> BinaryIO:
         return self.fs.open(self.path, mode="rb")
@@ -1215,10 +1222,17 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
         """
         Recursively copy this file or directory tree to the given destination.
         """
-        if not isinstance(target, UPath):
-            return super().copy(self.with_segments(target), **kwargs)
-        else:
-            return super().copy(target, **kwargs)
+        if isinstance(target, str):
+            proto = get_upath_protocol(target)
+            if proto != self.protocol:
+                target = UPath(target)
+            else:
+                target = self.with_segments(target)
+        elif not isinstance(target, UPath):
+            target = UPath(target)
+        if target.is_dir():
+            raise IsADirectoryError(str(target))
+        return super().copy(target, **kwargs)
 
     @overload
     def copy_into(self, target_dir: _WT, **kwargs: Any) -> _WT: ...
@@ -1232,10 +1246,19 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
         """
         Copy this file or directory tree into the given existing directory.
         """
-        if not isinstance(target_dir, UPath):
-            return super().copy_into(self.with_segments(target_dir), **kwargs)
-        else:
-            return super().copy_into(target_dir, **kwargs)
+        if isinstance(target_dir, str):
+            proto = get_upath_protocol(target_dir)
+            if proto != self.protocol:
+                target_dir = UPath(target_dir)
+            else:
+                target_dir = self.with_segments(target_dir)
+        elif not isinstance(target_dir, UPath):
+            target_dir = UPath(target_dir)
+        if not target_dir.exists():
+            raise FileNotFoundError(str(target_dir))
+        if not target_dir.is_dir():
+            raise NotADirectoryError(str(target_dir))
+        return super().copy_into(target_dir, **kwargs)
 
     @overload
     def move(self, target: _WT, **kwargs: Any) -> _WT: ...
@@ -1268,17 +1291,63 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
             raise ValueError(f"{self!r} has an empty name")
         elif hasattr(target_dir, "with_segments"):
             target = target_dir.with_segments(target_dir, name)  # type: ignore
+        elif isinstance(target_dir, PurePath):
+            target = UPath(target_dir, name)
         else:
             target = self.with_segments(target_dir, name)
+        td = target.parent
+        if not td.exists():
+            raise FileNotFoundError(str(td))
+        elif not td.is_dir():
+            raise NotADirectoryError(str(td))
         return self.move(target)
 
     def _copy_from(
         self,
         source: ReadablePath,
         follow_symlinks: bool = True,
+        on_name_collision: OnNameCollisionFunc | None = None,
         **kwargs: Any,
     ) -> None:
-        return super()._copy_from(source, follow_symlinks)
+        """
+        UPath custom:: Recursively copy the given path to this path.
+        """
+        # fixme: it would be best if this would be upstreamed
+        from pathlib_abc import vfsopen
+        from pathlib_abc import vfspath
+        from pathlib_abc._os import copyfileobj
+        from pathlib_abc._os import ensure_different_files
+
+        stack: list[tuple[ReadablePath, WritablePath]] = [(source, self)]
+        while stack:
+            src, dst = stack.pop()
+            info = src.info
+            if not follow_symlinks and info.is_symlink():
+                dst.symlink_to(vfspath(src.readlink()), src.info.is_dir())
+            elif on_name_collision and info.is_file() and info.is_dir():
+                dst_file, dst_dir = on_name_collision(src, dst)
+                if dst_file is not None:
+                    ensure_different_files(src, dst_file)
+                    with vfsopen(src, "rb") as source_f:
+                        with vfsopen(dst_file, "wb") as target_f:
+                            copyfileobj(source_f, target_f)
+                if dst_dir is not None:
+                    children = src.iterdir()
+                    dst_dir.mkdir()
+                    # feed through dict.fromkeys to remove duplicates
+                    for child in dict.fromkeys(children):
+                        stack.append((child, dst_dir.joinpath(child.name)))
+            elif info.is_dir():
+                children = src.iterdir()
+                dst.mkdir()
+                # feed through dict.fromkeys to remove duplicates
+                for child in dict.fromkeys(children):
+                    stack.append((child, dst.joinpath(child.name)))
+            else:
+                ensure_different_files(src, dst)
+                with vfsopen(src, "rb") as source_f:
+                    with vfsopen(dst, "wb") as target_f:
+                        copyfileobj(source_f, target_f)
 
     # --- WritablePath attributes -------------------------------------
 
@@ -1751,7 +1820,10 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
         if exists and not exist_ok:
             raise FileExistsError(str(self))
         if not exists:
-            self.fs.touch(self.path, truncate=True)
+            try:
+                self.fs.touch(self.path, truncate=True)
+            except NotImplementedError:
+                _raise_unsupported(type(self).__name__, "touch")
         else:
             try:
                 self.fs.touch(self.path, truncate=False)
@@ -1936,14 +2008,14 @@ class UPath(_UPathMixin, WritablePath, ReadablePath):
             args = (self.__vfspath__(),)
             kwargs = {
                 "protocol": self._protocol,
-                **self._storage_options,
+                **self.storage_options,
             }
         else:
             args = (self._relative_base, self.__vfspath__())
             # Include _relative_base in the state if it's set
             kwargs = {
                 "protocol": self._protocol,
-                **self._storage_options,
+                **self.storage_options,
                 "_relative_base": self._relative_base,
             }
         return _make_instance, (type(self), args, kwargs)
