@@ -2,9 +2,11 @@ from __future__ import annotations
 
 import enum
 import sys
+from collections.abc import Callable
 from os import PathLike
 from typing import TYPE_CHECKING
 from typing import Any
+from typing import Optional
 from typing import Protocol
 from typing import Union
 from typing import runtime_checkable
@@ -35,6 +37,7 @@ __all__ = [
     "PathParser",
     "UPathParser",
     "UNSET_DEFAULT",
+    "OnNameCollisionFunc",
 ]
 
 
@@ -124,3 +127,8 @@ class UPathParser(PathParser, Protocol):
     def splitdrive(self, path: JoinablePathLike) -> tuple[str, str]: ...
 
     def splitroot(self, path: JoinablePathLike) -> tuple[str, str, str]: ...
+
+
+OnNameCollisionFunc: TypeAlias = Callable[
+    [ReadablePath, WritablePath], tuple[Optional[WritablePath], Optional[WritablePath]]
+]
